@@ -112,7 +112,9 @@ test_backup_and_restore_helper (gboolean journal)
 
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           (const gchar **) test_schemas,
-	                           NULL, FALSE, 100, 100, NULL, NULL, NULL);
+	                           NULL, FALSE, 100, 100, NULL, NULL, NULL, &error);
+
+	g_assert_no_error (error);
 
 	/* load data set */
 	data_filename = g_strconcat (data_prefix, ".data", NULL);
@@ -168,7 +170,10 @@ test_backup_and_restore_helper (gboolean journal)
 
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           (const gchar **) test_schemas,
-	                           NULL, FALSE, 100, 100, NULL, NULL, NULL);
+	                           NULL, FALSE, 100, 100, NULL, NULL, NULL, &error);
+
+	g_assert_no_error (error);
+
 	check_content_in_db (0, 0);
 
 	tracker_data_backup_restore (backup_file, (const gchar **) test_schemas, NULL, NULL, &error);
